@@ -1,12 +1,16 @@
 #include "stdafx.h"
 
-void extension_build_params(PyObject *params_py, params *params) {
+ERROR_T extension_build_params(PyObject *params_py, params *params) {
     size_t index;
     char *key_s, *value_s, value_b;
     long value_i;
     float value_f;
     PyObject *iterator, *element, *key, *value;
 
+    if(!PySequence_Check(params_py)) {
+        RAISE_M("[extension_build_params] Invalid params data type");
+    }
+
     params->length = 0;
 
     index = 0;
@@ -82,6 +86,8 @@ void extension_build_params(PyObject *params_py, params *params) {
     }
 
     Py_DECREF(iterator);
+
+    NORMAL;
 };
 
 PyObject *extension_register(PyObject *self, PyObject *args) {
@@ -117,6 +123,8 @@ PyObject *extension_blend_images(PyObject *self, PyObject *args, PyObject *kwarg
     is_inline = NULL;
     params_py = NULL;
 
+    set_last_error(NULL);
+
     if(PyArg_ParseTupleAndKeywords(
         args,
         kwargs,
@@ -136,24 +144,38 @@ PyObject *extension_blend_images(PyObject *self, PyObject *args, PyObject *kwarg
     destination_over = strcmp(algorithm, "destination_over") == 0;
 
     if(params_py != NULL) {
-        extension_build_params(params_py, &params);
+        VALIDATE_PY(
+            extension_build_params(params_py, &params)
+        );
     }
 
     Py_BEGIN_ALLOW_THREADS;
-    VALIDATE_A(read_png(bottom_path, demultiply, &bottom), Py_BLOCK_THREADS Py_RETURN_NONE);
-    VALIDATE_A(read_png(top_path, demultiply, &top), Py_BLOCK_THREADS Py_RETURN_NONE);
+    VALIDATE_PY(read_png(bottom_path, demultiply, &bottom), Py_BLOCK_THREADS);
+    VALIDATE_PY(read_png(top_path, demultiply, &top), Py_BLOCK_THREADS);
     if(source_over == TRUE) {
-        VALIDATE_A(blend_images_source_over_fast(&bottom, &top, algorithm, &params), Py_BLOCK_THREADS Py_RETURN_NONE);
+        VALIDATE_PY(
+            blend_images_source_over_fast(&bottom, &top, algorithm, &params),
+            Py_BLOCK_THREADS
+        );
     } else if(destination_over == TRUE) {
-        VALIDATE_A(blend_images_destination_over_fast(&bottom, &top, algorithm, &params), Py_BLOCK_THREADS Py_RETURN_NONE);
+        VALIDATE_PY(
+            blend_images_destination_over_fast(&bottom, &top, algorithm, &params),
+            Py_BLOCK_THREADS
+        );
     } else if(run_inline == TRUE && source_over == TRUE) {
-        VALIDATE_A(blend_images_i(&bottom, &top, algorithm, &params), Py_BLOCK_THREADS Py_RETURN_NONE);
+        VALIDATE_PY(
+            blend_images_i(&bottom, &top, algorithm, &params),
+            Py_BLOCK_THREADS
+        );
     } else {
-        VALIDATE_A(blend_images(&bottom, &top, algorithm, &params), Py_BLOCK_THREADS Py_RETURN_NONE);
+        VALIDATE_PY(
+            blend_images(&bottom, &top, algorithm, &params),
+            Py_BLOCK_THREADS
+        );
     }
-    VALIDATE_A(write_png(&bottom, demultiply, target_path), Py_BLOCK_THREADS Py_RETURN_NONE);
-    VALIDATE_A(release_image(&top), Py_BLOCK_THREADS Py_RETURN_NONE);
-    VALIDATE_A(release_image(&bottom), Py_BLOCK_THREADS Py_RETURN_NONE);
+    VALIDATE_PY(write_png(&bottom, demultiply, target_path), Py_BLOCK_THREADS);
+    VALIDATE_PY(release_image(&top), Py_BLOCK_THREADS);
+    VALIDATE_PY(release_image(&bottom), Py_BLOCK_THREADS);
     Py_END_ALLOW_THREADS;
 
     Py_RETURN_NONE;
@@ -190,6 +212,8 @@ PyObject *extension_blend_multiple(PyObject *self, PyObject *args, PyObject *kwa
     is_inline = NULL;
     params_py = NULL;
 
+    set_last_error(NULL);
+
     if(PyArg_ParseTupleAndKeywords(
         args,
         kwargs,
@@ -219,7 +243,7 @@ PyObject *extension_blend_multiple(PyObject *self, PyObject *args, PyObject *kwa
     /* in case the parameters value has been provided, then it must be parsed
     as a list of dictionaries containing the parameters */
     if(params_py != NULL) {
-        extension_build_params(params_py, &params);
+        VALIDATE_PY(extension_build_params(params_py, &params));
     }
 
     /* retrieves the size of the paths sequence that has been provided
@@ -231,7 +255,11 @@ PyObject *extension_blend_multiple(PyObject *self, PyObject *args, PyObject *kwa
     if(use_algorithms) {
         algorithms_size = PySequence_Size(algorithms);
         if(algorithms_size != size - 1) {
-            abort_("[extension_blend_multiple] Invalid algorithms list size");
+            PyErr_SetString(
+                PyExc_TypeError,
+                "[extension_blend_multiple] Invalid algorithms list size"
+            );
+            return NULL;
         }
     }
 
@@ -248,17 +276,17 @@ PyObject *extension_blend_multiple(PyObject *self, PyObject *args, PyObject *kwa
         bottom_path = PyString_AsString(first);
 #endif
         Py_BEGIN_ALLOW_THREADS;
-        VALIDATE_A(
+        VALIDATE_PY(
             read_png(bottom_path, demultiply, &bottom),
-            Py_BLOCK_THREADS Py_RETURN_NONE
+            Py_BLOCK_THREADS
         );
-        VALIDATE_A(
+        VALIDATE_PY(
             write_png(&bottom, demultiply, target_path),
-            Py_BLOCK_THREADS Py_RETURN_NONE
+            Py_BLOCK_THREADS
         );
-        VALIDATE_A(
+        VALIDATE_PY(
             release_image(&bottom),
-            Py_BLOCK_THREADS Py_RETURN_NONE
+            Py_BLOCK_THREADS
         );
         Py_END_ALLOW_THREADS;
 #if PY_MAJOR_VERSION >= 3
@@ -323,37 +351,37 @@ PyObject *extension_blend_multiple(PyObject *self, PyObject *args, PyObject *kwa
     /* validates that both the bottom and the top path are correctly
     read from the current file system, in case an error occurs return
     an invalid (none) value to the caller python code */
-    VALIDATE_A(
+    VALIDATE_PY(
         read_png(bottom_path, demultiply, &bottom),
-        Py_BLOCK_THREADS Py_RETURN_NONE
+        Py_BLOCK_THREADS
     );
-    VALIDATE_A(
+    VALIDATE_PY(
         read_png(top_path, demultiply, &top),
-        Py_BLOCK_THREADS Py_RETURN_NONE
+        Py_BLOCK_THREADS
     );
 
     if(source_over == TRUE) {
-        VALIDATE_A(
+        VALIDATE_PY(
             blend_images_source_over_fast(&bottom, &top, algorithm, &params),
-            Py_BLOCK_THREADS Py_RETURN_NONE
+            Py_BLOCK_THREADS
         );
     } else if(destination_over == TRUE) {
-        VALIDATE_A(
+        VALIDATE_PY(
             blend_images_destination_over_fast(&bottom, &top, algorithm, &params),
-            Py_BLOCK_THREADS Py_RETURN_NONE
+            Py_BLOCK_THREADS
         );
     } else if(run_inline == TRUE && source_over == TRUE) {
-        VALIDATE_A(
+        VALIDATE_PY(
             blend_images_i(&bottom, &top, algorithm, &params),
-            Py_BLOCK_THREADS Py_RETURN_NONE
+            Py_BLOCK_THREADS
         );
     } else {
-        VALIDATE_A(
+        VALIDATE_PY(
             blend_images(&bottom, &top, algorithm, &params),
-            Py_BLOCK_THREADS Py_RETURN_NONE
+            Py_BLOCK_THREADS
         );
     }
-    VALIDATE_A(release_image(&top), Py_BLOCK_THREADS Py_RETURN_NONE);
+    VALIDATE_PY(release_image(&top), Py_BLOCK_THREADS);
     Py_END_ALLOW_THREADS;
 
 #if PY_MAJOR_VERSION >= 3
@@ -420,32 +448,32 @@ PyObject *extension_blend_multiple(PyObject *self, PyObject *args, PyObject *kwa
 
         Py_BEGIN_ALLOW_THREADS;
 
-        VALIDATE_A(
+        VALIDATE_PY(
             read_png(top_path, demultiply, &top),
-            Py_BLOCK_THREADS Py_RETURN_NONE
+            Py_BLOCK_THREADS
         );
         if(source_over == TRUE) {
-            VALIDATE_A(
+            VALIDATE_PY(
                 blend_images_source_over_fast(&bottom, &top, algorithm, &params),
-                Py_BLOCK_THREADS Py_RETURN_NONE
+                Py_BLOCK_THREADS
             );
         } else if(destination_over == TRUE) {
-            VALIDATE_A(
+            VALIDATE_PY(
                 blend_images_destination_over_fast(&bottom, &top, algorithm, &params),
-                Py_BLOCK_THREADS Py_RETURN_NONE
+                Py_BLOCK_THREADS
             );
         } else if(run_inline == TRUE && source_over == TRUE) {
-            VALIDATE_A(
+            VALIDATE_PY(
                 blend_images_i(&bottom, &top, algorithm, &params),
-                Py_BLOCK_THREADS Py_RETURN_NONE
+                Py_BLOCK_THREADS
             );
         } else {
-            VALIDATE_A(
+            VALIDATE_PY(
                 blend_images(&bottom, &top, algorithm, &params),
-                Py_BLOCK_THREADS Py_RETURN_NONE
+                Py_BLOCK_THREADS
             );
         }
-        VALIDATE_A(release_image(&top), Py_BLOCK_THREADS Py_RETURN_NONE);
+        VALIDATE_PY(release_image(&top), Py_BLOCK_THREADS);
         Py_END_ALLOW_THREADS;
         Py_DECREF(element);
         if(use_algorithms) { Py_DECREF(algorithm_o); }
@@ -458,13 +486,13 @@ PyObject *extension_blend_multiple(PyObject *self, PyObject *args, PyObject *kwa
     Py_DECREF(iterator);
 
     Py_BEGIN_ALLOW_THREADS;
-    VALIDATE_A(
+    VALIDATE_PY(
         write_png(&bottom, demultiply, target_path),
-        Py_BLOCK_THREADS Py_RETURN_NONE
+        Py_BLOCK_THREADS
     );
-    VALIDATE_A(
+    VALIDATE_PY(
         release_image(&bottom),
-        Py_BLOCK_THREADS Py_RETURN_NONE
+        Py_BLOCK_THREADS
     );
     Py_END_ALLOW_THREADS;
 
