@@ -2,7 +2,7 @@
 
 #include "structs.h"
 
-#define PCONVERT_VERSION "0.3.5"
+#define PCONVERT_VERSION "0.3.6"
 #define PCONVERT_COMPILATION_DATE __DATE__
 #define PCONVERT_COMPILATION_TIME __TIME__
 #define PCONVERT_ALGORITHMS "multiplicative", "source_over", "destination_over",\
@@ -20,17 +20,38 @@
 #define NO_ERROR 0
 #define RAISE return ERROR
 #define RAISE_S(...) abort_(__VA_ARGS__); return ERROR
+#define RAISE_M(message) set_last_error(message); return ERROR
+#define RAISE_F(message, ...) set_last_error_f(message, __VA_ARGS__); return ERROR
+#define EXCEPT_S(input) if(IS_ERROR(input)) {\
+    print_(last_error == NULL ? "Unknown error" : last_error);\
+    return ERROR;\
+} while(FALSE)
+#define EXCEPT_M(input, ...) if(IS_ERROR(input)) {\
+    __VA_ARGS__;\
+    print_(last_error == NULL ? "Unknown error" : last_error);\
+    return ERROR;\
+} while(FALSE)
 #define NORMAL return NO_ERROR
 #define IS_ERROR(input) input != NO_ERROR
 #define VALIDATE(input) if(IS_ERROR(input)) { RAISE; } while(FALSE)
 #define VALIDATE_R(input, ...) if(IS_ERROR(input)) { return __VA_ARGS__; } while(FALSE)
 #define VALIDATE_A(input, ...) if(IS_ERROR(input)) { __VA_ARGS__; } while(FALSE)
+#define VALIDATE_PY(input, ...) if(IS_ERROR(input)) {\
+    __VA_ARGS__;\
+    PyErr_SetString(PyExc_TypeError, last_error == NULL ? "Unknown error" : last_error);\
+    return NULL;\
+} while(FALSE)
 
 #define Z_NO_COMPRESSION 0
 #define Z_BEST_SPEED 1
 #define Z_BEST_COMPRESSION 9
 #define Z_DEFAULT_COMPRESSION (-1)
 
+#define MAX_ERROR_L 1024
+
+EXTERNAL_PREFIX char *last_error;
+EXTERNAL_PREFIX char last_error_b[MAX_ERROR_L];
+
 typedef struct pcv_image {
     int width;
     int height;
@@ -50,6 +71,7 @@ typedef void (blend_algorithm) (
 
 void print_(const char *s, ...);
 void abort_(const char *s, ...);
+void set_last_error_f(char *message, ...);
 ERROR_T read_png(char *file_name, char demultiply, struct pcv_image *image);
 ERROR_T write_png(struct pcv_image *image, char multiply, char *file_name);
 ERROR_T write_png_extra(
@@ -205,3 +227,7 @@ static FINLINE void blend_source_over_i(
     *(result + 2) = b;
     *(result + 3) = a;
 }
+
+static FINLINE void set_last_error(char *message, ...) {
+    last_error = message;
+}
